@@ -33,12 +33,9 @@
 
   #:use-module (ice-9 curried-definitions)
   #:use-module (ice-9 match)
-  #:use-module (ice-9 pretty-print)
-  #:use-module (ice-9 receive)
 
   #:use-module (dzn ast goops)
   #:use-module (dzn ast)
-  #:use-module (dzn command-line)
   #:use-module (dzn misc)
 
   #:export (%normalize:short-circuit?
@@ -327,7 +324,7 @@
          (expression (simplify-expression expression)))
     (if (or (is-a? expression <unary>)
             (is-a? expression <field-test>)) expression
-        (clone o #:expression expression))))
+            (clone o #:expression expression))))
 
 (define-method (simplify-expression (o <expression>))
   o)
@@ -548,12 +545,14 @@ guarded occurrences."
   (define ((trigger-equal? trigger) triple)
     (let ((t ((compose car ast:trigger* triple-on) triple)))
       (and (equal? (.port.name t) (.port.name trigger)) (equal? (.event.name t) (.event.name trigger)))))
-  (let* ((sorted-triples (let loop ((triples triples))
-                           (if (null? triples) '()
-                               (let ((trigger ((compose car ast:trigger* triple-on car) triples)))
-                                 (receive (shared rest)
-                                     (partition (trigger-equal? trigger) triples)
-                                   (cons shared (loop rest)))))))
+  (let* ((sorted-triples
+          (let loop ((triples triples))
+            (if (null? triples) '()
+                (let* ((trigger ((compose car ast:trigger* triple-on car)
+                                 triples))
+                       (shared rest
+                               (partition (trigger-equal? trigger) triples)))
+                  (cons shared (loop rest))))))
          (ons (map
                (lambda (triples)
                  (let* ((on ((compose triple-on car) triples))
@@ -739,14 +738,16 @@ to prevent unintended shadowing
   (match o
     ((? (is? <ast-list>))
      (clone o #:elements (filter-map purge-data (.elements o))))
-    (($ <data>) #f)
+    (($ <data>)
+     #f)
     (($ <action>)
      (clone o #:arguments (make <arguments>)))
-
+    (($ <call>)
+     (clone o #:arguments (purge-data (.arguments o))))
     (($ <trigger>)
      (clone o #:formals (make <formals>)))
-
-    (($ <extern>) #f)
+    (($ <extern>)
+     #f)
     (($ <assign>)
      (let* ((variable (.variable o))
             (type (and variable (.type variable))))
@@ -755,18 +756,16 @@ to prevent unintended shadowing
     (($ <formal>)
      (let ((type (.type o)))
        (and type (not (is-a? type <extern>)) o)))
-    (($ <call>) (clone o #:arguments (make <arguments> #:elements (filter (negate is-data?) (ast:argument* o)))))
     (($ <variable>)
      (let ((type (.type o)))
        (and type (not (is-a? type <extern>))
             (clone o #:expression (purge-data (.expression o))))))
-
     (($ <var>)
      (let* ((variable (.variable o))
             (type (and variable (.type variable))))
        (and type (not (is-a? type <extern>)) o)))
-
-    ((and ($ <return>) (= .expression ($ <data-expr>))) (clone o #:expression #f))
+    ((and ($ <return>) (= .expression ($ <data-expr>)))
+     (clone o #:expression #f))
     ((? (%normalize:short-circuit?))
      o)
     (($ <interface>)
@@ -840,7 +839,7 @@ to prevent unintended shadowing
   (match o
     (($ <defer>)
      (let ((statement (add-defer-end (.statement o))))
-      (clone o #:statement (add-end statement))))
+       (clone o #:statement (add-end statement))))
     (($ <blocking>)
      (clone o #:statement (add-defer-end (.statement o))))
     (($ <on>)
@@ -881,7 +880,7 @@ to prevent unintended shadowing
     (($ <guard>) (clone o #:statement (add-reply-port (.statement o) port block?)))
     (($ <compound>) (clone o #:elements (map (cut add-reply-port <> port block?) (ast:statement* o))))
     (($ <behavior>) (clone o #:statement (add-reply-port (.statement o) port block?)
-                            #:functions (add-reply-port (.functions o) port block?)))
+                           #:functions (add-reply-port (.functions o) port block?)))
     ((? (%normalize:short-circuit?)) o)
     (($ <component>) (clone o #:behavior (add-reply-port (.behavior o) (if (= 1 (length (ast:provides-port* o))) (car (ast:provides-port* o)) #f) block?)))
     (($ <interface>) o)
@@ -1128,7 +1127,7 @@ expressions explicit."
 (define (add-determinism-temporaries o)
   "Make evaluation order of noisy expressions deterministic by adding
 explitic temporaries."
-    (parameterize ((%noisy-ordering? #t))
+  (parameterize ((%noisy-ordering? #t))
     (add-explicit-temporaries o)))
 
 (define (split-complex-expressions o)
