@@ -1,6 +1,6 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023 Janneke Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2021 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2017, 2018, 2019 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2018 Paul Hoogendijk <paul@dezyne.org>
@@ -26,26 +26,15 @@
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-26)
 
-  #:use-module (ice-9 match)
-  #:use-module (ice-9 regex)
-  #:use-module (ice-9 getopt-long)
-  #:use-module (ice-9 pretty-print)
-
-  #:use-module (dzn misc)
-
-  #:use-module ((oop goops) #:renamer (lambda (x) (if (member x '(<port> <foreign>)) (symbol-append 'goops: x) x)))
-  #:use-module (dzn goops)
-  #:use-module (dzn config)
-
+  #:use-module (dzn ast goops)
+  #:use-module (dzn ast)
   #:use-module (dzn code dzn)
   #:use-module (dzn code)
   #:use-module (dzn code-util)
-  #:use-module (dzn indent)
-  #:use-module (dzn ast)
+  #:use-module (dzn config)
+  #:use-module (dzn misc)
   #:use-module (dzn templates)
 
-  #:declarative? #f
-
   #:export (c++:capture-arguments
             c++:dzn-locator
             c++:enum->string
@@ -53,6 +42,7 @@
             c++:enum-field-type
             c++:enum-literal
             c++:formal-type
+            c++:include-guard
             c++:model
             c++:string->enum-field*
             c++:type-name
@@ -81,21 +71,17 @@
 (define-method (c++:enum->string (o <interface>))
   (filter (is? <enum>) (ast:type* o)))
 
-(define-method (c++:type-name o)
-  (let* ((type (or (as o <model>) (as o <type>) (ast:type o))))
-    (match type
-      (($ <enum>) (c++:type-name type))
-      (($ <extern>) (list (.value type)))
-      (($ <bool>) '("bool"))
-      (($ <int>) '("int"))
-      (($ <void>) '("void"))
-      (_ (c++:type-name type)))))
+(define-method (c++:enum->string (o <component>))
+  (filter (is? <enum>) (ast:type* (.behavior o))))
+
+(define-method (c++:enum->string (o <enum>))
+  o)
 
 (define-method (c++:type-name o)
   (code:type-name o))
 
 (define-method (c++:type-name (o <enum>))
-  (append (list "") (ast:full-name o) (list "type")))
+  (append (list "") (ast:full-name o)))
 
 (define-method (c++:type-name (o <enum-field>))
   (append (c++:type-name (.type o)) (list (.field o))))
@@ -125,21 +111,22 @@
   (let* ((models (ast:model* o))
          (models (filter (negate
                           (disjoin (is? <type>) (is? <namespace>)
-                                   ast:async?
                                    ast:imported?))
-                      models))
+                         models))
          (models (ast:topological-model-sort models))
          (models (map code:annotate-shells models)))
     models))
 
 (define-method (c++:dzn-locator (o <instance>))
-  (let ((model (parent o <model>)))
+  (let ((model (ast:parent o <model>)))
     (if (null? (code:injected-bindings model)) '()
         o)))
 
-(define-method (c++:pump-for-async? (o <component>))
-  (if ((compose pair? ast:req-events) o) o
-      '()))
+(define-method (c++:include-guard (o <model>))
+  o)
+
+(define-method (c++:include-guard (o <instance>))
+  (.type o))
 
 (define-templates-macro define-templates c++)
 (include-from-path "dzn/templates/dzn.scm")
@@ -156,7 +143,7 @@
 
   (code-util:foreign-conflict? root)
 
-  (let ((root (code:normalize+determinism root)))
+  (let ((root (code:om+determinism root)))
     (let ((generator (code-util:indenter (cute x:header root)))
           (file-name (code-util:root-file-name root dir ".hh")))
       (code-util:dump root generator #:file-name file-name))
