@@ -4,7 +4,7 @@
 ;;; Copyright © 2019, 2020 Johri van Eerd <vaneerd.johri@gmail.com>
 ;;; Copyright © 2018, 2019, 2020 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2018, 2019, 2020 Paul Hoogendijk <paul@dezyne.org>
-;;; Copyright © 2018, 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2018, 2019, 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -26,35 +26,32 @@
 ;;; Code:
 
 (define-module (dzn code makreel)
-  #:use-module (ice-9 curried-definitions)
-  #:use-module (ice-9 hash-table)
-  #:use-module (ice-9 match)
-  #:use-module (ice-9 poe)
-  #:use-module (ice-9 pretty-print)
-  #:use-module (ice-9 receive)
-  #:use-module (ice-9 textual-ports)
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-26)
 
-  #:use-module (dzn config)
-  #:use-module (dzn command-line)
-
-  #:use-module ((oop goops) #:renamer (lambda (x) (if (member x '(<port> <foreign>)) (symbol-append 'goops: x) x)))
-  #:use-module (dzn goops)
-  #:use-module (dzn display)
-  #:use-module (dzn misc)
+  #:use-module (ice-9 curried-definitions)
+  #:use-module (ice-9 match)
+  #:use-module (ice-9 receive)
 
+  #:use-module (dzn ast display)
+  #:use-module (dzn ast equal)
+  #:use-module (dzn ast goops)
+  #:use-module (dzn ast normalize)
   #:use-module (dzn ast)
-
-  #:use-module (dzn code)
   #:use-module (dzn code dzn)
-  #:use-module (dzn normalize)
+  #:use-module (dzn code)
+  #:use-module (dzn command-line)
+  #:use-module (dzn config)
+  #:use-module (dzn misc)
   #:use-module (dzn templates)
+  #:use-module (dzn verify constraint)
+
   #:export (%model-name
             makreel:.name
             makreel:enum-fields
             makreel:get-model
             makreel:init-process
+            makreel:line-column
             makreel:model->makreel
             makreel:model-name
             makreel:name
@@ -62,6 +59,7 @@
             makreel:reply-type-sort
             makreel:tick-names
             makreel:unticked-dotted-name
+            makreel:constraint->makreel
             root->))
 
 (define %id-alist (make-parameter #f))
@@ -116,8 +114,8 @@
   (tree-collect (is? <defer>) o))
 
 (define-method (makreel:interface-reorder (o <behavior>))
-  (if (parent o <interface>) o
-      '()))
+  (and (ast:parent o <interface>)
+       o))
 
 (define-method (makreel:interface-reorder (o <ast>))
   '())
@@ -134,13 +132,13 @@
           (string-append "%% no cache hit found: \n" intf)))))
 
 (define-method (x:interface-proc-memo (o <interface>))
- (makreel:interface-proc-memo o))
+  (makreel:interface-proc-memo o))
 
 (define-method (x:interface-proc-memo (o <component>))
- (string-join (map makreel:interface-proc-memo  (ast:interface* o)) "\n"))
+  (string-join (map makreel:interface-proc-memo (makreel:interface* o)) "\n"))
 
 (define (mcrl2:process-identifier o)
-  (let* ((model-key ((compose .id (cut parent <> <model>)) o))
+  (let* ((model-key ((compose .id (cute ast:parent <> <model>)) o))
          (path (ast:path o))
          (key (map .id path))
 	 (next (assq-ref (%next-alist) model-key))
@@ -163,13 +161,13 @@
 
 (define (last-statement? o)
   (or (not (is-a? (.parent o) <compound>))
-      (let* ((p (parent o <scope>))
-             (statements (ast:statement* p)))
+      (let* ((parent(ast:parent o <scope>))
+             (statements (ast:statement* parent)))
         (ast:eq? (last statements) o))))
 
 (define-method (makreel:assign-call-parameter (o <variable>))
-  (if (last-statement? o) '()
-      o))
+  (and (not (last-statement? o))
+       o))
 
 (define-method (makreel:assign-call-parameter (o <assign>))
   (if (and (last-statement? o)
@@ -194,9 +192,10 @@
   (match o
     (($ <root>) (tree-map (tick-names-) o))
     ((? (is? <model>)) (clone (tree-map (tick-names-) o) #:name ((compose (tick-names-) .name) o)))
+    (($ <int>) o)
     (($ <bool>) o)
     (($ <void>) o)
-    ((and ($ <scope.name>) (or (= .ids '("void")) (= .ids '("bool")))) o)
+    ((and ($ <scope.name>) (or (= .ids '("<int>")) (= .ids '("void")) (= .ids '("bool")))) o)
     ((? (is? <type>)) (clone o #:name ((compose (tick-names-) .name) o)))
     (($ <scope.name>) (clone o #:ids (map (append-tick) (.ids o))))
     (($ <port>) (clone o #:name ((compose (append-tick) .name) o) #:type.name ((compose (tick-names-) .type.name) o)))
@@ -266,7 +265,7 @@
     (($ <call>) (let ((continuation ((compose car makreel:continuation) o)))
                   (if (and (is-a? continuation <return>)
                            (is-a? (ast:type (.expression continuation)) <void>)) (clone o #:last? #t)
-                      o)))
+                           o)))
     (($ <expression>) o)
     (($ <location>) o)
     ((? (is? <ast>)) (tree-map makreel:mark-tail-call o))
@@ -282,6 +281,10 @@
           (find (is? <component>) (ast:model* o))
           (find (is? <interface>) (ast:model* o))))))
 
+;;HACK: TODO REMOVEME
+(define-method (makreel:interface-name (o <component>))
+  (makreel:interface-name (car (ast:port* o))))
+
 (define-method (makreel:interface-name (o <interface>))
   (makreel:model-name o))
 
@@ -289,19 +292,26 @@
   ((compose makreel:interface-name .type) o))
 
 (define-method (makreel:interface-name (o <on>))
-  (makreel:interface-name (or (parent o <interface>)
+  (makreel:interface-name (or (ast:parent o <interface>)
                               (.type (.port (car (ast:trigger* o)))))))
 
 (define-method (makreel:interface-name (o <event>))
-  (makreel:interface-name (parent o <interface>)))
+  (makreel:interface-name (ast:parent o <interface>)))
 
 (define-method (makreel:interface-name (o <statement>))
-  (makreel:interface-name (parent o <on>)))
+  (makreel:interface-name (ast:parent o <on>)))
 
 (define-method (makreel:interface-name (o <action>))
   (makreel:interface-name
-   (or (.port o) (parent o <on>)
-       (let ((model (parent o <model>)))
+   (or (.port o) (ast:parent o <on>)
+       (let ((model (ast:parent o <model>)))
+         (if (is-a? model <interface>) model
+             ((compose .type car ast:provides-port*) o))))))
+
+(define-method (makreel:interface-name (o <trigger>))
+  (makreel:interface-name
+   (or (.port o) (ast:parent o <on>)
+       (let ((model (ast:parent o <model>)))
          (if (is-a? model <interface>) model
              ((compose .type car ast:provides-port*) o))))))
 
@@ -309,10 +319,10 @@
   (string-join (ast:full-name o) ""))
 
 (define-method (makreel:model-name (o <ast>))
-  (makreel:model-name (parent o <model>)))
+  (makreel:model-name (ast:parent o <model>)))
 
 (define-method (makreel:scope-name (o <ast>))
-  (and=> (parent o <model>) makreel:model-name))
+  (and=> (ast:parent o <model>) makreel:model-name))
 
 (define-method (makreel:multiple-provides? (o <port>))
   (if (< 1 (length (ast:provides-port* o))) o
@@ -326,11 +336,10 @@
   (delete-duplicates (map .type (filter ast:provides? (ast:port* o)))))
 
 (define-method (makreel:flush-provides-ports (o <port>))
-  (ast:provides-port* (parent o <component>)))
+  (ast:provides-port* (ast:parent o <component>)))
 
-(define-method (ast:async-port*? (o <component>))
-  (if (pair? (ast:async-port* o)) o
-      '()))
+(define-method (makreel:port-variable* (port <port>))
+  (ast:variable* (.behavior (.type port))))
 
 (define-method (ast:non-external-port* (o <component>))
   (filter (negate ast:external?) (filter ast:requires? (ast:port* o))))
@@ -339,7 +348,7 @@
   (filter ast:external? (ast:port* o)))
 
 (define-method (makreel:action-sort (o <component>))
-  (ast:interface* o))
+  (makreel:interface* o))
 
 (define-method (makreel:action-sort (o <interface>))
   o)
@@ -349,27 +358,27 @@
 
 (define-method (makreel:action-sort-event (o <port>))
   (filter (compose (cut equal? (.name o) <>) .port.name)
-          (ast:trigger* (parent o <component>))))
+          (ast:trigger* (ast:parent o <component>))))
 
 (define-method (enum-sort-global-public (o <root>))
   (filter (is? <enum>)
-    (append
-      (ast:type* o)
-      (append-map ast:type* (filter (is? <interface>) (ast:model* o))))))
+          (append
+           (ast:type* o)
+           (append-map ast:type* (filter (is? <interface>) (ast:model* o))))))
 
 (define-method (makreel:enum-sort (o <interface>))
   (append
-    (enum-sort-global-public (parent o <root>))
-    (filter (is? <enum>) (ast:type* (.behavior o)))))
+   (enum-sort-global-public (ast:parent o <root>))
+   (filter (is? <enum>) (ast:type* (.behavior o)))))
 
 (define-method (makreel:enum-sort (o <component>))
   (append
-    (enum-sort-global-public (parent o <root>))
-    (append-map
-      (lambda (o) (filter (is? <enum>) (ast:type* (.behavior o))))
-      (append (list o) (ast:interface* o)))))
+   (enum-sort-global-public (ast:parent o <root>))
+   (append-map
+    (lambda (o) (filter (is? <enum>) (ast:type* (.behavior o))))
+    (append (list o) (makreel:interface* o)))))
 
-(define-method (makreel:reply-type-eq? (a <int>) (b <int>))
+(define-method (makreel:reply-type-eq? (a <subint>) (b <subint>))
   #t)
 (define-method (makreel:reply-type-eq? a b)
   (ast:eq? a b))
@@ -388,6 +397,7 @@
     (match type
       (($ <bool>) o)
       (($ <int>) o)
+      (($ <subint>) o)
       (($ <void>) o)
       (_ type))))
 
@@ -395,7 +405,7 @@
   (makreel:type-constructor (.port o)))
 
 (define-method (makreel:modeling-sort (o <component>))
-  (ast:interface* o))
+  (makreel:interface* o))
 
 (define-method (makreel:modeling-sort (o <interface>))
   o)
@@ -407,27 +417,25 @@
   ((compose makreel:event-type-name .event) o))
 
 (define (reachable calls)
-  (receive (nested direct) (partition (cut parent <> <function>) calls)
+  (receive (nested direct) (partition (cute ast:parent <> <function>) calls)
     (let loop ((nested nested) (direct direct))
       (receive (reached rest) (partition
                                (lambda (call)
-                                 (find (cut ast:eq? (parent call <function>) <>)
+                                 (find (cut ast:eq? (ast:parent call <function>) <>)
                                        (map .function direct))) nested)
         (let* ((reached (append reached direct)))
           (if (equal? direct reached) direct
               (loop rest reached)))))))
 
-(define (reachable-calls- o)
+(define (reachable-calls-unmemoized root o)
   (let* ((calls (tree-collect-filter
-                  (disjoin (is? <behavior>) (is? <declarative>) (is? <functions>) (is? <function>) (is? <statement>))
-                  (is? <call>) o))
+                 (disjoin (is? <behavior>) (is? <declarative>) (is? <functions>) (is? <function>) (is? <statement>))
+                 (is? <call>) o))
          (calls (reachable calls)))
     calls))
 
 (define (reachable-calls o)
-  (define (calls root o)
-    (reachable-calls- o))
- ((ast:pure-funcq calls) (parent o <root>) o))
+  ((ast:pure-funcq reachable-calls-unmemoized) (ast:parent o <root>) o))
 
 (define-method (no-tail-call (o <call>))
   (not (.last? o)))
@@ -444,25 +452,26 @@
   (call-continuations o #f))
 
 (define-method (makreel:recurse? (o <call>))
-  (if (.last? o) o
-      '()))
+  (and (.last? o)
+       o))
 
 (define-method (makreel:non-recurse? (o <call>))
-  (if (.last? o) '()
-      o))
+  (and (not (.last? o))
+       o))
 
 (define-method (makreel:function-return-proc (o <model>))
-  (if (pair? ((compose makreel:called-function* .behavior) o)) o
-      '()))
+  (and (pair? ((compose makreel:called-function* .behavior) o))
+       o))
 
 (define-method (makreel:function-return (o <model>))
-  (append-map makreel:function-return ((compose makreel:called-function* .behavior) o)))
+  (append-map makreel:function-return
+              ((compose makreel:called-function* .behavior) o)))
 
 (define-method (makreel:function-return (o <function>))
-  (call-continuations (parent o <behavior>) (.name o)))
+  (call-continuations (ast:parent o <behavior>) (.name o)))
 
 (define-method (is-called? (o <function>))
-  (let ((calls (reachable-calls (parent o <behavior>))))
+  (let ((calls (reachable-calls (ast:parent o <behavior>))))
     (find (compose (cut equal? <> (.name o)) .function.name) calls)))
 
 (define-method (makreel:called-function* (o <behavior>))
@@ -476,8 +485,8 @@
       (clone o #:expression (make <void>))))
 
 (define-method (makreel:return-type-sort (o <model>))
-  (if (pair? ((compose makreel:called-function* .behavior) o)) o
-      '()))
+  (and (pair? ((compose makreel:called-function* .behavior) o))
+       o))
 
 (define-method (ast:return-type-eq? (a <function>) (b <function>) )
   (apply ast:eq? (map (compose .type .signature) (list a b))))
@@ -496,17 +505,11 @@
   (or (and (find ast:requires? (ast:port* o)) o) '()))
 
 (define-method (ast:have-requires? (o <port>))
-  (ast:have-requires? (parent o <component>)))
+  (ast:have-requires? (ast:parent o <component>)))
 
 (define-method (ast:have-no-requires? (o <component>))
   (or (and (not (find ast:requires? (ast:port* o))) o) '()))
 
-(define-method (ast:have-requires+async? (o <component>))
-  (if (pair? (ast:requires+async-port* o)) o '()))
-
-(define-method (ast:have-no-requires+async? (o <component>))
-  (if (pair? (ast:requires+async-port* o)) '() o))
-
 (define-method (makreel:queue-length (o <component>))
   (%queue-size))
 
@@ -518,7 +521,7 @@
 
 (define-method (makreel:event-act (o <component>))
   (append
-   (ast:interface* o)
+   (makreel:interface* o)
    (ast:port* o)
    (ast:port* (.behavior o))))
 
@@ -535,6 +538,9 @@
   (if (dzn:command-line:get 'debug) "\n%behavior"
       ""))
 
+(define-method (pretty-print-dzn (o <top>))
+  "")
+
 (define-method (pretty-print-dzn (o <statement>))
   (let ((debug? (dzn:command-line:get 'debug)))
     (if debug? (string-join (string-split (string-trim-right
@@ -544,30 +550,35 @@
 
 
 (define-method (makreel:event-prefix (o <on>))
-  (let ((model (parent o <model>)))
+  (let ((model (ast:parent o <model>)))
     (if (is-a? model <interface>) model
         (.port (car (ast:trigger* o))))))
 
 (define-method (makreel:event-prefix (o <reply>))
   (or (.port o)
-      (let ((model (parent o <model>)))
+      (let ((model (ast:parent o <model>)))
         (if (is-a? model <interface>) model
-            (let ((trigger (and=> (parent o <on>) (compose car ast:trigger*))))
-              (if (and trigger (ast:provides? trigger)) ((compose .port) trigger)
-                  ((compose car ast:provides-port*) model)))))))
+            (begin
+              (let ((trigger (and=> (ast:parent o <on>) (compose car ast:trigger*))))
+                (if (and trigger (ast:provides? trigger)) ((compose .port) trigger)
+                    ((compose car ast:provides-port*) model))))))))
 
 (define-method (makreel:event-prefix (o <port>))
   o)
 
 (define-method (makreel:event-prefix (o <action>))
   (let ((port (.port o)))
-    (or port (parent o <model>))))
+    (or port (ast:parent o <model>))))
 
 (define-method (makreel:event-prefix (o <assign>))
   (ast:port-type-name o))
 
 (define-method (makreel:event-prefix (o <statement>))
-  (makreel:event-prefix (parent o <on>)))
+  (makreel:event-prefix (ast:parent o <on>)))
+
+(define-method (makreel:event-prefix (o <trigger>))
+  (let ((port (.port o)))
+    (or port (ast:parent o <model>))))
 
 (define-method (makreel:trigger-name (o <on>))
   (let ((trigger ((compose car ast:trigger*) o)))
@@ -576,8 +587,23 @@
 (define-method (makreel:interface-proc (o <interface>))
   o)
 
+(define-method (makreel:interface* (o <component>))
+  (delete-duplicates
+   (append
+    (map .type (ast:port* o))
+    (filter (conjoin (negate ast:imported?) (is? <interface>))
+            (ast:model* (ast:parent o <root>))))
+   ast:eq?))
+
+(define-method (ast:provides-interface* (o <component>))
+  (delete-duplicates (append (map .type (ast:provides-port* o))) ast:eq?))
+
+(define-method (makreel:unique-provides-port* (o <component>))
+  (delete-duplicates (ast:provides-port* o)
+                     (match-lambda* ((a b) (ast:eq? (.type a) (.type b))))))
+
 (define-method (makreel:interface-proc (o <component>))
-  (ast:interface* o))
+  (makreel:interface* o))
 
 (define-method (is-optional? (o <guard>))
   (is-optional? (.statement o)))
@@ -604,13 +630,13 @@
   (let ((expression (.expression o)))
     (if (or (is-a? expression <action>)
             (is-a? expression <call>)) expression
-        o)))
+            o)))
 
 (define-method (makreel:proc-variable (o <variable>))
   (let ((expression (.expression o)))
     (if (or (is-a? expression <action>)
             (is-a? expression <call>)) expression
-        o)))
+            o)))
 
 (define-method (makreel:else-proc (o <if>))
   (or (.else o) '()))
@@ -619,6 +645,10 @@
   (let ((in? (eq? 'in (.direction (.event o)))))
     (if in? o '())))
 
+(define-method (makreel:reply-synchronization (o <trigger>))
+  (let ((in? (eq? 'in (.direction (.event o)))))
+    (if in? o '())))
+
 (define (unspecified? x) (eq? x *unspecified*))
 
 (define-method (makreel:on-proc (o <on>))
@@ -643,33 +673,34 @@
 (define-method (makreel:process-index (o <behavior>))
   ((compose makreel:process-index .statement) o))
 
-(define-method (members (o <ast>))
-  (let* ((parent (parent o <model>))
-	 (behavior (.behavior parent)))
-    (ast:variable* behavior)))
+(define-method (members (o <model>))
+  (ast:member* o))
 
-(define-method (makreel:locals- (o <ast>))
+(define-method (members (o <ast>))
+  (members (ast:parent o <model>)))
+
+(define-method (makreel:locals-unmemoized root (o <ast>))
   (if (is-a? o <behavior>) '()
       (let* ((p (.parent o)))
         (cond ((is-a? p <compound>)
                (let ((pre (cdr (member o (reverse (ast:statement* p)) ast:eq?))))
                  (append (filter (is? <variable>) pre) (makreel:locals p))))
               ((is-a? p <defer>)
-               (makreel:locals- p))
+               (let ((model (ast:parent p <model>)))
+                 (makreel:locals p)))
               ((is-a? o <function>) ((compose ast:formal* .signature) o))
               (else (makreel:locals p))))))
 
 (define (makreel:locals o)
-  (define (locals root o)
-    (makreel:locals- o))
-  (let* ((model (parent o <model>))
-         (root (parent model <root>)))
-    ((ast:pure-funcq locals) (list root model) o)))
+  (let* ((model (ast:parent o <model>))
+         (root (ast:parent model <root>)))
+    ((ast:pure-funcq makreel:locals-unmemoized) (list root model) o)))
 
 (define-method (variables-in-scope (o <model>)) (members o))
 (define-method (variables-in-scope (o <ast>))
-  (let ((stack (and (parent o <function>)
-                    (not (parent (.parent o) <defer>))
+  (let ((stack (and (or (as o <function>)
+                        (ast:parent o <function>))
+                    (not (ast:parent o <defer>))
                     (clone (make <stack>) #:parent o))))
     (append (members o) (makreel:locals o) (if stack (list stack) '()))))
 
@@ -678,13 +709,13 @@
 
 (define-method (makreel:variable-parameter (o <ast>))
   (let ((v (match o
-               (($ <variable>) o)
-               (($ <assign>) (.variable o)))))
+             (($ <variable>) o)
+             (($ <assign>) (.variable o)))))
     (if (find (cut ast:eq? v <>) ((compose variables-in-scope car makreel:continuation) o)) o
         '())))
 
 (define-method (makreel:function-name (o <ast>))
-  (or (and=> (parent o <function>) .name) '()))
+  (or (and=> (ast:parent o <function>) .name) '()))
 
 (define-method (makreel:process-parameters (o <ast>))
   (variables-in-scope o))
@@ -707,26 +738,28 @@
 
 (define-method (makreel:process-haakjes (o <ast>))
   (if (or (pair? (variables-in-scope o))
-          (parent o <function>)) "()"
+          (ast:parent o <function>)) "()"
           ""))
 
 (define-method (makreel:continuation-haakjes (o <ast>))
-  (if (or (pair? ((compose variables-in-scope car makreel:continuation) o))
-          (parent o <function>)) o
-          '()))
+  (and (or (pair? ((compose variables-in-scope car makreel:continuation) o))
+           (ast:parent o <function>))
+       o))
 
 (define-method (makreel:process-continuation (o <ast>))
   (mcrl2:process-continuation o))
 
 (define-method (makreel:sum-helper-params (o <ast>))
   (let* ((locals (variables-in-scope o))
-         (var (list (or (parent o <variable>) (.variable (parent o <assign>)))))
+         (var (list (or (ast:parent o <variable>)
+                        (.variable (ast:parent o <assign>)))))
          (params (append locals var)))
     (delete-duplicates params ast:eq?)))
 
 (define-method (makreel:the-end (o <ast>))
-  (if (and (parent o <component>) (ast:eq? o (.statement (parent o <behavior>)))) o
-      '()))
+  (and (ast:parent o <component>)
+       (ast:eq? o (.statement (ast:parent o <behavior>)))
+       o))
 
 (define-method (makreel:continuation (o <ast>))
   (define (statement-continuation o)
@@ -736,15 +769,14 @@
           (let ((grandp (.parent p)))
             (match grandp
               (($ <compound>) (statement-continuation p))
-              (($ <defer>) (list (parent o <behavior>)))
-              ((? (is? <declarative>)) (list (parent o <behavior>)))
+              (($ <defer>) (list (ast:parent o <behavior>)))
+              ((? (is? <declarative>)) (list (ast:parent o <behavior>)))
               (_  (makreel:continuation grandp)))))))
 
   (let* ((cont
           (match o
             ((and ($ <declarative-compound>) (= ast:statement* ())) (throw 'barf "unexpected empty declarative-compound"))
             ((and ($ <declarative-compound>) (= ast:statement* elements)) elements)
-            ;;ASSUME normalized model <guard> <on> <compound>
             (($ <behavior>) (list (.statement o)))
             (($ <function>) (list (.statement o)))
             (($ <guard>) (list (.statement o)))
@@ -757,7 +789,7 @@
              (let* ((p (.parent o)))
                (match p
                  (($ <compound>) (statement-continuation o))
-                 ((? (is? <declarative>)) (list (parent o <behavior>)))
+                 ((? (is? <declarative>)) (list (ast:parent o <behavior>)))
                  (($ <on>) (throw 'barf "unexpected on"))
                  (_ (makreel:continuation p)))))
             (_ (makreel:continuation (.parent o))))))
@@ -827,6 +859,18 @@
 (define-method (makreel:provides-reply (o <port>))
   (ast:provides-port* o))
 
+(define-method (makreel:provides-reply-init (o <port-pair>))
+  (let* ((other (.other o))
+         (interface (makreel:interface-name (.port o))))
+    (if (ast:eq? other (.port o)) (format #f "~anil" interface)
+        other)))
+
+(define-method (makreel:provides-reply-init (o <port>))
+  (ast:provides-port* o))
+
+(define-method (makreel:provides-reply-init (o <component>))
+  (ast:provides-port* o))
+
 (define-method (makreel:provides-reset-reply (o <port-pair>))
   (let ((port (.port o))
         (other (.other o)))
@@ -834,12 +878,12 @@
         other)))
 
 (define-method (makreel:rename-flush-provides (o <port>))
-  (if (ast:provides? o) o
-      '()))
+  (and (ast:provides? o)
+       o))
 
 (define-method (makreel:rename-flush-requires (o <port>))
-  (if (ast:requires? o) o
-      '()))
+  (and (ast:requires? o)
+       o))
 
 (define-method (makreel:allow-tau (o <component>))
   (delete-duplicates (append (map .type (ast:port* o))) ast:eq?))
@@ -851,7 +895,7 @@
   o)
 
 (define-method (makreel:switch-context (o <action>))
-  (let* ((component (parent o <component>))
+  (let* ((component (ast:parent o <component>))
          (blocking? (and component
                          (ast:requires? o)
                          (find ast:blocking? (ast:requires-port* component)))))
@@ -865,12 +909,12 @@
 
 (define-method (ast:port-type-name (o <reply>)) ;; FIXME: return AST (interface/port) rather than string
   (let ((interface
-            (or (and=> (.port o) .type)
-                (let ((model (parent o <model>)))
-                  (if (is-a? model <interface>) model
-                      (let ((trigger (and=> (parent o <on>) (compose car ast:trigger*))))
-                        (if (and trigger (ast:provides? trigger)) ((compose .type .port) trigger)
-                            ((compose .type car ast:provides-port*) model))))))))
+         (or (and=> (.port o) .type)
+             (let ((model (ast:parent o <model>)))
+               (if (is-a? model <interface>) model
+                   (let ((trigger (and=> (ast:parent o <on>) (compose car ast:trigger*))))
+                     (if (and trigger (ast:provides? trigger)) ((compose .type .port) trigger)
+                         ((compose .type car ast:provides-port*) model))))))))
     (makreel:model-name interface)))
 
 (define-method (ast:port-type-name (o <assign>))
@@ -878,18 +922,18 @@
     (match expression
       (($ <action>)
        (let ((port (.port .expression)))
-         (or port (parent o <model>))))
+         (or port (ast:parent o <model>))))
       (_ "BOO"))))
 
 (define-method (ast:port-type-name (o <action>))
   (let ((port (.port o)))
-    (or port (parent o <model>))))
+    (or port (ast:parent o <model>))))
 
 (define-method (ast:port-type-name (o <port>))
   ((compose makreel:model-name .type) o))
 
 (define-method (ast:port-type-name (o <expression>))
-  (ast:port-type-name (parent o <reply>)))
+  (ast:port-type-name (ast:parent o <reply>)))
 
 (define-method (.event.name (o <assign>))
   (let ((expression (.expression o)))
@@ -907,22 +951,24 @@
        (ast:field* o)))
 
 (define-method (makreel:type-bound (o <action>))
-  (if (is-a? ((compose .type .signature .event) o) <int>) o
-      '()))
+  (and (is-a? ((compose .type .signature .event) o) <subint>)
+       o))
 
 (define-method (makreel:type-check (o <action>))
-  (if (is-a? ((compose .type .signature .event) o) <int>)
-      (let ((parent (.parent o)))
-        (match parent
-          (($ <assign>) (.variable parent))
-          (($ <variable>) parent)))
-      '()))
+  (and (is-a? ((compose .type .signature .event) o) <subint>)
+       (let* ((parent (.parent o))
+              (variable (match parent
+                          (($ <assign>) (.variable parent))
+                          (($ <variable>) parent)))
+              (type (ast:type variable)))
+         (and (is-a? type <subint>)
+              variable))))
 
 (define (as-int o)
-  (or (and o (is-a? (ast:type o) <int>) o) '()))
+  (and o (is-a? (ast:expression->type o) <subint>) o))
 
 (define-method (makreel:type-check (o <call>))
-  (filter (compose (is? <int>) ast:type) (ast:argument* o)))
+  (filter (compose (is? <subint>) ast:expression->type) (ast:argument* o)))
 
 (define-method (makreel:type-check (o <return>))
   (as-int (.expression o)))
@@ -934,27 +980,38 @@
   (as-int (.expression o)))
 
 (define-method (makreel:type-check (o <model>))
-  (map .expression (filter (compose (is? <int>) .type) (ast:member* o))))
+  (map .expression (filter (compose (is? <subint>) .type) (ast:member* o))))
 
 (define-method (makreel:stack? (o <call>))
-  (or (and (not (parent o <defer>)) (parent o <function>) o) '()))
+  (or (and (not (is-a? o <defer>))
+           (not (ast:parent o <defer>))
+           (or (is-a? o <function>)
+               (ast:parent o <function>))
+           o)
+      '()))
 
 (define-method (makreel:stack-empty? (o <call>))
-  (or (and (or (parent o <defer>) (not (parent o <function>))) o) '()))
+  (or (and (or (is-a? o <defer>)
+               (ast:parent o <defer>)
+               (not (ast:parent o <function>)))
+           o)
+      '()))
 
 (define-method (makreel:stack-destructor (o <ast>))
-  (let ((f (and (not (parent o <defer>))
-                (parent o <function>)))
+  (let ((f (and (not (is-a? o <defer>))
+                (not (ast:parent o <defer>))
+                (or (is-a? o <function>)
+                    (ast:parent o <function>))))
         (locals (makreel:locals o))
         (return-value (if (and (or (is-a? o <assign>) (is-a? o <variable>))
                                (is-a? (.expression o) <call>)) (list (ast:type o))
-                          '())))
+                               '())))
     (if f (append locals (list f)
                   return-value)
         (append locals return-value))))
 
 (define-method (makreel:process-argument-stack? (o <call>))
-  (let ((function (parent o <function>)))
+  (let ((function (ast:parent o <function>)))
     (cond ((and function (.last? o)) function)
           (else o))))
 
@@ -964,24 +1021,85 @@
 (define (makreel:init-process process)
   (format #f "init ~a;\n" process))
 
+(define-method (makreel:line-column (o <tag>))
+  (let ((location (.location o)))
+    (format #f "~a, ~a" (.line location) (.column location))))
+
+
+;;;
+;;; Constraint
+;;;
+(define (makreel:constraint-unmemoized o)
+  (let ((root (ast:parent o <root>)))
+    (interface->constraint root o)))
+
+(define makreel:constraint
+  (let ((cache-alist '()))
+    (lambda (o)
+      (let ((key (makreel:unticked-dotted-name o)))
+        (or (assoc-ref cache-alist key)
+            (let ((result (makreel:constraint-unmemoized o)))
+              (set! cache-alist (acons key result cache-alist))
+              result))))))
+
+(define-method (makreel:provides-port-name (o <component>))
+  (.name (ast:provides-port o)))
+
+(define-method (makreel:single-provides-component (o <component>))
+  (and (= (length (ast:provides-port* o)) 1)
+       o))
+
+(define-method (makreel:multiple-provides-component (o <component>))
+  (and (> (length (ast:provides-port* o)) 1)
+       o))
+
+(define-method (makreel:constraint-continuation (o <on>))
+  (and (or (is-a? (ast:parent o <model>) <interface>)
+           (ast:requires? (car (ast:trigger* o))))
+       o))
+
+(define-method (makreel:constraint-continuation-illegal (o <on>))
+  (and (is-a? (ast:parent o <model>) <component>)
+       (ast:provides? (car (ast:trigger* o)))
+       o))
+
+(define-method (makreel:constraint-component-illegal (o <on>))
+  (and (not (is-a? (ast:parent o <model>) <interface>))
+       o))
+
+(define-method (makreel:semantics-provides-flush (o <component>))
+  (and (pair? (ast:requires-port* o))
+       o))
+
 (define-templates-macro define-templates makreel)
 (include-from-path "dzn/templates/dzn.scm")
 (include-from-path "dzn/templates/makreel.scm")
 
+(define (makreel:short-circuit? o)
+  (match o
+    ((or ($ <foreign>) ($ <system>))
+     o)
+    ((and ($ <component>) (? ast:imported?))
+     o)
+    (_
+     #f)))
+
 (define (makreel:om ast)
-  (let ((root ((compose
-                makreel:mark-tail-call
-                add-function-return
-                normalize:state+illegals
-                remove-otherwise
-                makreel:tick-names
-                add-explicit-temporaries
-                add-defer-end
-                purge-data
-                ) ast)))
-    (when (> (dzn:debugity) 1)
-      (ast:pretty-print root (current-error-port)))
-    root))
+  (parameterize ((%normalize:short-circuit? makreel:short-circuit?))
+    (let ((root ((compose
+                  makreel:mark-tail-call
+                  add-function-return
+                  normalize:state+illegals
+                  remove-otherwise
+                  makreel:tick-names
+                  add-explicit-temporaries
+                  add-defer-end
+                  (if (%no-unreachable?) identity tag-imperative-blocks)
+                  purge-data
+                  ) ast)))
+      (when (> (dzn:debugity) 1)
+        (ast:pretty-print root (current-error-port)))
+      root)))
 
 
 ;;;
@@ -994,6 +1112,9 @@
     (newline)))
 
 (define* (ast-> ast #:key dir model)
-  (let ((root (makreel:om ast)))
+  (let ((root (makreel:om ast))
+        (init (command-line:get 'init)))
     (if model (makreel:model->makreel root (makreel:get-model root model))
-        (root-> root))))
+        (root-> root))
+    (when init
+      (display (makreel:init-process init)))))
