@@ -1,6 +1,6 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2014, 2015, 2017, 2019, 2020, 2021, 2022, 2023 Janneke Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2014, 2015, 2017, 2019, 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2017 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2019 Rob Wieringa <rma.wieringa@gmail.com>
 ;;;
@@ -24,23 +24,22 @@
 ;;; Code:
 
 (define-module (dzn code scheme)
-  #:use-module (ice-9 receive)
-  #:use-module (ice-9 match)
-  #:use-module (ice-9 string-fun)
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-26)
 
-  #:use-module (dzn config)
-  #:use-module (dzn misc)
-
-  #:use-module ((oop goops) #:renamer (lambda (x) (if (member x '(<port> <foreign>)) (symbol-append 'goops: x) x)))
-  #:use-module (dzn goops)
+  #:use-module (ice-9 receive)
+  #:use-module (ice-9 match)
+  #:use-module (ice-9 string-fun)
 
+  #:use-module (dzn ast goops)
   #:use-module (dzn ast)
+  #:use-module (dzn ast normalize)
   #:use-module (dzn code dzn)
   #:use-module (dzn code)
   #:use-module (dzn code-util)
+  #:use-module (dzn config)
   #:use-module (dzn indent)
+  #:use-module (dzn misc)
   #:use-module (dzn templates))
 
 (define (string->class-name o)
@@ -53,7 +52,7 @@
   (string-join (ast:full-name o) ":"))
 
 (define-method (scheme:class-name (o <ast>))
-  (scheme:class-name (parent o <model>)))
+  (scheme:class-name (ast:parent o <model>)))
 
 ;; Work around a bug that base name of a Guile module cannot include
 ;; dots.  See http://debbugs.gnu.org/cgi/bugreport.cgi?bug=39162
@@ -109,7 +108,7 @@
                                  (ast:port* o))
                             (if (or (is-a? o <foreign>)
                                     (not (.behavior o))) '()
-                                (map (compose string->accessor .name) (ast:variable* o))))))
+                                    (map (compose string->accessor .name) (ast:variable* o))))))
     (append classes accessors)))
 
 (define-method (scheme:names (o <system>))
@@ -187,7 +186,7 @@
     (map (cut make <file-name> #:name <>) files)))
 
 (define-method (scheme:use-module (o <model>))
-  (scheme:use-module (parent o <root>)))
+  (scheme:use-module (ast:parent o <root>)))
 
 (define-method (scheme:variable/local (o <formal>))
   (if (ast:in? o) o
@@ -200,14 +199,6 @@
 (define-method (scheme:set! (o <assign>))
   (scheme:variable/local (.variable o)))
 
-(define-method (scheme:async-req (o <port>))
-  (let ((event (find (cut ast:name-equal? <> "req") (ast:event* (.type o)))))
-    (make <trigger> #:port.name (.name o) #:event.name (.name event) #:formals (ast:rescope ((compose .formals .signature) event) (parent o <model>)))))
-
-(define-method (scheme:async-clr (o <port>))
-  (let ((event (find (cut ast:name-equal? <> "clr") (ast:event* (.type o)))))
-    (make <trigger> #:port.name (.name o) #:event.name (.name event) #:formals (ast:rescope ((compose .formals .signature) event)  (parent o <model>)))))
-
 (define (comment-mangler str prefix)
   (let* ((str (string-replace-substring str "/*" ""))
          (str (string-replace-substring str "*/" ""))
@@ -229,17 +220,17 @@
          (make <compound> #:elements (list o))))
     (($ <behavior>) (clone o #:statement (wrap-lonely-variable (.statement o))))
     (($ <component>) (clone o #:behavior (wrap-lonely-variable (.behavior o))))
-    (($ <system>) o)
-    (($ <foreign>) o)
     (($ <interface>) (clone o #:behavior (wrap-lonely-variable (.behavior o))))
+    ((? (%normalize:short-circuit?)) o)
     ((? (is? <ast>)) (tree-map wrap-lonely-variable o))
     (_ o)))
 
-(define (scheme:normalize ast)
-  ((compose
-    wrap-lonely-variable
-    code:normalize)
-   ast))
+(define (scheme:om ast)
+  (parameterize ((%normalize:short-circuit? code:short-circuit?))
+    ((compose
+      wrap-lonely-variable
+      code:om)
+     ast)))
 
 (define-templates-macro define-templates scheme)
 (include-from-path "dzn/templates/dzn.scm")
@@ -256,7 +247,7 @@
 
   (code-util:foreign-conflict? root)
 
-  (let ((root (scheme:normalize root))
+  (let ((root (scheme:om root))
         (indenter (cute code-util:indenter <>
                         #:open #\( #:close #\) #:no-indent "")))
 
