@@ -27,21 +27,18 @@
 (define-module (dzn commands parse)
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-26)
+  #:use-module (srfi srfi-71)
   #:use-module (ice-9 getopt-long)
   #:use-module (ice-9 pretty-print)
   #:use-module (ice-9 rdelim)
-  #:use-module (ice-9 regex)
-
-  #:use-module (dzn serialize)
 
+  #:use-module (dzn ast serialize)
+  #:use-module (dzn ast display)
+  #:use-module (dzn ast)
   #:use-module (dzn command-line)
-  #:use-module (dzn misc)
-  #:use-module (dzn parse)
   #:use-module (dzn parse peg)
   #:use-module (dzn parse tree)
-  #:use-module (dzn shell-util)
-  #:use-module (dzn ast)
-  #:use-module (dzn wfc)
+  #:use-module (dzn parse)
 
   #:export (parse
             parse-opts
@@ -79,7 +76,7 @@ Parse a Dezyne file and produce an AST
   -t, --parse-tree       write PEG parse tree
   -o, --output=FILE      write AST to FILE (\"-\" for standard output)
 ")
-          (exit (or (and usage? EXIT_OTHER_FAILURE) EXIT_SUCCESS))))
+        (exit (or (and usage? EXIT_OTHER_FAILURE) EXIT_SUCCESS))))
     options))
 
 (define (parse options file-name)
@@ -89,19 +86,24 @@ Parse a Dezyne file and produce an AST
          (locations? (command-line:get 'locations))
          (model-name (option-ref options 'model #f))
          (parse-tree? (command-line:get 'parse-tree))
-         (fall-back? (command-line:get 'fall-back)))
+         (fall-back? (command-line:get 'fall-back))
+         (transform (dzn:multi-opt 'transform)))
     (parameterize ((%locations? locations?)
-                   (%peg:fall-back? fall-back?))
-      (let ((ast (file->ast file-name
-                            #:debug? debug?
-                            #:imports imports
-                            #:parse-tree? parse-tree?
-                            #:skip-wfc? skip-wfc?)))
-        (if (not model-name) ast
-            (call-with-handle-exceptions
-             (lambda _ (ast:filter-model ast (ast:get-model ast model-name)))
-             #:backtrace? debug?
-             #:file-name file-name))))))
+                   (%peg:fall-back? fall-back?)
+                   (%peg:error (format-display-syntax-error file-name)))
+      (let* ((ast parse-failed? (file->ast file-name
+                                           #:debug? debug?
+                                           #:imports imports
+                                           #:parse-tree? parse-tree?
+                                           #:skip-wfc? skip-wfc?
+                                           #:transform transform))
+             (ast (if (not model-name) ast
+                      (call-with-handle-exceptions
+                       (lambda _
+                         (ast:filter-model ast (ast:get-model ast model-name)))
+                       #:backtrace? debug?
+                       #:file-name file-name))))
+        (values ast parse-failed?)))))
 
 (define (preprocess options file-name)
   (let* ((imports (command-line:get 'import))
@@ -132,20 +134,26 @@ Parse a Dezyne file and produce an AST
          (files (option-ref options '() '()))
          (file-name (and (pair? files) (car files)))
          (list-models? (command-line:get 'list-models))
+         (debug? (dzn:command-line:get 'debug))
          (preprocess? (option-ref options 'preprocess #f)))
     (cond (preprocess?
            (display (preprocess options file-name)))
           (list-models? (list-models file-name))
           (else
-           (let ((ast (parse options file-name)))
+           (let ((ast parse-failed? (parse options file-name)))
              (if (option-ref options 'output #f)
                  (let* ((file-name (option-ref options 'output "-"))
                         (locations? (command-line:get 'locations))
-                        (sexp (parameterize ((%locations? locations?))
-                                (ast:serialize ast)))
+                        (sexp (and (not debug?)
+                                   (parameterize ((%locations? locations?))
+                                     (ast:serialize ast))))
                         (output (with-output-to-string
-                                      (cute pretty-print sexp))))
+                                  (if debug? (cute ast:pretty-print ast)
+                                      (cute pretty-print sexp)))))
                    (if (equal? file-name "-") (display output)
                        (with-output-to-file file-name (cut display output))))
                  (when (dzn:command-line:get 'verbose)
-                   (display "parse: no errors found\n"))))))))
+                   (if parse-failed? (display "parse: errors found\n")
+                       (display "parse: no errors found\n"))))
+             (when parse-failed?
+               (exit EXIT_FAILURE)))))))
