@@ -1,6 +1,6 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2017, 2018, 2019, 2020, 2021, 2022, 2023 Janneke Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2017, 2018, 2019, 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2018, 2020, 2021, 2022 Paul Hoogendijk <paul@dezyne.org>
 ;;; Copyright © 2017, 2018, 2021, 2022, 2023 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2017, 2018, 2019 Rob Wieringa <rma.wieringa@gmail.com>
@@ -27,21 +27,20 @@
 (define-module (dzn commands traces)
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-26)
+
   #:use-module (ice-9 getopt-long)
   #:use-module (ice-9 regex)
-  #:use-module ((oop goops) #:renamer (lambda (x) (if (member x '(<port> <foreign>)) 'goops:<port> x)))
 
-  #:use-module (dzn misc)
-  #:use-module (dzn config)
-  #:use-module (dzn goops)
+  #:use-module (dzn ast goops)
   #:use-module (dzn ast)
-  #:use-module (dzn commands parse)
-  #:use-module (dzn command-line)
-  #:use-module (dzn lts)
+  #:use-module (dzn code)
   #:use-module (dzn code makreel)
-  #:use-module (dzn parse)
+  #:use-module (dzn command-line)
+  #:use-module (dzn commands parse)
+  #:use-module (dzn config)
+  #:use-module (dzn lts)
+  #:use-module (dzn misc)
   #:use-module (dzn shell-util)
-
   #:use-module (dzn verify pipeline)
 
   #:export (parse-opts
@@ -54,9 +53,12 @@
             (help (single-char #\h))
             (illegal (single-char #\i))
             (import (single-char #\I) (value #t))
+            (jitty (single-char #\j))
             (lts (single-char #\l))
             (traces (single-char #\t))
             (model (single-char #\m) (value #t))
+            (no-constraint (single-char #\C))
+            (no-non-compliance (single-char #\D))
             (output (single-char #\o) (value #t))
             (queue-size (single-char #\q) (value #t))
             (queue-size-defer (value #t))
@@ -71,9 +73,12 @@
 Usage: dzn traces [OPTION]... DZN-FILE
 Generate exhaustive set of traces for Dezyne model
 
+  -C, --no-constraint         do not use a constraining process
+  -D, --no-non-compliance     report deadlock upon non-compliance
   -f, --flush                 include <flush> events in trace
   -h, --help                  display this help and exit
   -i, --illegal               include traces that lead to an illegal
+  -j, --jitty                 pass to lps2lts, for LARGE models
   -I, --import=DIR+           add DIR to import path
   -l, --lts                   also generate LTS
   -m, --model=MODEL           generate traces for model MODEL
@@ -88,26 +93,20 @@ Generate exhaustive set of traces for Dezyne model
         (exit (or (and usage? EXIT_OTHER_FAILURE) EXIT_SUCCESS))))
     options))
 
-(define (mark-async text async-ports)
-  (let loop ((text text) (async-ports async-ports))
-    (if (null? async-ports) text
-        (loop (regexp-substitute/global #f (string-append "\"" (makreel:.name  (car async-ports)) ".inevitable\"") text 'pre "\"<ack>\"" 'post)
-              (cdr async-ports)))))
-
-(define (remove-mark-async text)
-  (regexp-substitute/global #f "\"<ack>\"" text 'pre "\"tau\"" 'post))
-
 (define (lts-hide-internal-labels text)
   (let* ((text (regexp-substitute/global #f "\"<declarative-illegal>\"" text 'pre "\"<illegal>\"" 'post))
          (text (regexp-substitute/global #f "\"[^\"]*<blocking>\"" text 'pre "\"tau\"" 'post))
          (text (regexp-substitute/global #f "\"[^\"]*\\.qout\\.[^\"]*\"" text 'pre "\"tau\"" 'post))
          (text (regexp-substitute/global #f "\"(optional|inevitable)\"" text 'pre "\"tau\"" 'post))
-         (text (regexp-substitute/global #f "\"[^\"]*\\.(optional|inevitable)\"" text 'pre "\"tau\"" 'post)))
+         (text (regexp-substitute/global #f "\"[^\"]*\\.(optional|inevitable)\"" text 'pre "\"tau\"" 'post))
+         (text (regexp-substitute/global #f "\"tag[()].[^\"]*\"" text 'pre "\"tau\"" 'post)))
     text))
 
 (define (model->lts root model file-name)
-  (let* ((lts (verify-pipeline "aut-weak-trace" root model))
-         (lts (if (is-a? model <interface>) lts (mark-async lts (ast:async-port* model))))
+  (let* ((jitty? (command-line:get 'jitty))
+         (lts (verify-pipeline (if jitty? "aut-weak-trace-jitty"
+                                   "aut-weak-trace")
+                               root model))
          (lts (lts-hide-internal-labels lts)))
     (when (string-null? (string-trim-right lts))
       (throw 'error "failed to create LTS"))
@@ -147,7 +146,7 @@ Generate exhaustive set of traces for Dezyne model
     (when lts?
       (if (and output (not (equal? output "-")))
           (with-output-to-file (string-append output "/" model-name ".aut")
-            (cute display (remove-mark-async lts)))
+            (cute display lts))
           (display lts)))))
 
 (define (main args)
@@ -155,7 +154,6 @@ Generate exhaustive set of traces for Dezyne model
          (files (option-ref options '() '()))
          (file-name (car files))
          (ast (parse options file-name))
-         (root (makreel:normalize ast))
          (model-name (option-ref options 'model #f))
          (queue-size (option-ref options 'queue-size (%queue-size)))
          (queue-size-defer (option-ref options 'queue-size-defer
@@ -164,15 +162,37 @@ Generate exhaustive set of traces for Dezyne model
                                           (%queue-size-external))))
     (define (named? o)
       (equal? (makreel:unticked-dotted-name o) model-name))
-    (parameterize ((%queue-size queue-size)
+    (parameterize ((%no-unreachable? #t)
+                   (%queue-size queue-size)
                    (%queue-size-defer queue-size-defer)
                    (%queue-size-external queue-size-external))
-    (let* ((models (ast:model* root))
-           (components-interfaces (append (filter (conjoin (is? <component>) .behavior) models)
-                                          (filter (is? <interface>) models)))
-           (model (or (and model-name (find named? models))
-                      (and (pair? components-interfaces) (car components-interfaces)))))
-      (cond ((and model-name (not model)) (error "no such model:" model-name))
-            ((is-a? model <system>) #t) ;; silently no traces
-            ((and model-name (or (is-a? model <foreign>) (not (.behavior model)))) (error "no model with behavior:" model-name))
-            (model (model->traces options root model file-name)))))))
+      (let* ((root (makreel:om ast))
+             (models (ast:model* root))
+             (components-interfaces
+              (append
+               (filter (conjoin (is? <component>) .behavior) models)
+               (filter (is? <interface>) models)))
+             (model (or (and model-name (find named? models))
+                        (and (pair? components-interfaces)
+                             (car components-interfaces)))))
+        (when (and=> model ast:imported?)
+          (let ((name (ast:dotted-name model)))
+            (format (current-error-port)
+                    "~a:error: cannot generate traces for imported model: ~a\n"
+                    (ast:source-file root)
+                    name)
+            (format (current-error-port)
+                    "~a:info: ~a imported from here\n"
+                    (ast:source-file model)
+                    name))
+          (exit EXIT_OTHER_FAILURE))
+        (cond ((and model-name (not model))
+               (error "no such model:" model-name))
+              ((is-a? model <system>) ;; silently no traces
+               #t)
+              ((and model-name
+                    (or (is-a? model <foreign>)
+                        (not (.behavior model))))
+               (error "no model with behavior:" model-name))
+              (model
+               (model->traces options root model file-name)))))))
