@@ -2,7 +2,7 @@
 ;;;
 ;;; Copyright © 2018, 2019 Henk Katerberg <hank@mudball.nl>
 ;;; Copyright © 2018, 2019, 2020, 2021, 2022, 2023 Paul Hoogendijk <paul@dezyne.org>
-;;; Copyright © 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2019, 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
@@ -46,7 +46,6 @@
 
   #:use-module (ice-9 match)
   #:use-module (ice-9 poe)
-  #:use-module (ice-9 optargs)
   #:use-module (ice-9 rdelim)
   #:use-module (ice-9 regex)
 
@@ -341,15 +340,14 @@
             (trace-it (edge-from parent) (cons parent edges)))))
     (trace-it state '())))
 
-(define (tau-loop loop-entry-state lts)
-  "Loop of events leading from LOOP-ENTRY-STATE back to LOOP-ENTRY-STATE"
+(define (tau-loop state lts)
+  "Loop of events leading from STATE back to STATE"
   (define (loop-edges trace state)
-    (let* ((predecessor-edge (node-cycle (vector-ref lts state)))
-           (predecessor-state (edge-from predecessor-edge))
-           (predecessor-node (vector-ref lts predecessor-state)))
-      (if (eq? predecessor-state loop-entry-state) (cons predecessor-edge trace)
-          (loop-edges (cons predecessor-edge trace) predecessor-state))))
-  (loop-edges '()  loop-entry-state))
+    (let* ((edge (node-cycle (vector-ref lts state)))
+           (state (edge-from edge)))
+      (if (find (compose (cute eq? <> state) edge-from) trace) trace
+          (loop-edges (cons edge trace) state))))
+  (loop-edges '() state))
 
 (define (assert-livelock lts)
   "Trace to entry point of first livelock or #f if no tau loops found"
@@ -386,25 +384,6 @@
   (let ((lts (annotate-exclude lts (list %<declarative-illegal> %<illegal>))))
     (vector-map-one exclude lts)))
 
-(define (trim-queue-full lts)
-  (define (reset-exclude node)
-    (set-field node (node-color) #f))
-  (define (exclude-queue-full node)
-    (let ((edges (if (node-exclude? node) '()
-                     (node-edges node))))
-      (set-field node (node-edges) edges)))
-  (define (set-exclude-queue-full lts i node)
-    (let* ((edges (node-edges node))
-           (edge (find (compose (cute eq? <> %<queue-full>) edge-label) edges)))
-      (when edge
-        (let* ((i (edge-to edge))
-               (node (vector-ref lts i))
-               (node (set-field node (node-color) #t)))
-          (vector-set! lts i node)))))
-  (let ((lts (vector-map-one reset-exclude lts)))
-    (vector-for-each (cute set-exclude-queue-full lts <> <>) lts)
-    (vector-map-one exclude-queue-full lts)))
-
 
 ;;;
 ;;; Deadlock.
@@ -420,7 +399,6 @@
 (define (deadlock-nodes lts)
   "States without outgoing edges"
   (let* ((lts (remove-state-edges lts))
-         (lts (trim-queue-full lts))
          (lts (annotate-exclude lts (list %<declarative-illegal>))))
     (define (edges? state)
       (let ((node (vector-ref lts state)))
