@@ -1,8 +1,8 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2019, 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2018, 2019 Rob Wieringa <rma.wieringa@gmail.com>
-;;; Copyright © 2018, 2019, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2018, 2019, 2021, 2022, 2023 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2021, 2022 Paul Hoogendijk <paul@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
@@ -32,9 +32,7 @@
   #:use-module (ice-9 string-fun)
 
   #:use-module (dzn ast goops)
-  #:use-module (dzn ast serialize)
   #:use-module (dzn ast)
-  #:use-module (dzn command-line)
   #:use-module (dzn config)
   #:use-module (dzn misc)
   #:use-module (dzn vm ast)
@@ -44,7 +42,6 @@
   #:export (%debug
             %debug?
             %next-input
-            %queue-size
             %startup-info
             %strict?
 
@@ -157,9 +154,6 @@
   (when (%debug?)
     (format (current-error-port) fmt arg ...)))
 
-;;; The size of the component queues and external queues.
-(define %queue-size (make-parameter 3))
-
 ;; Is the input trail to be matched exactly?
 (define %strict? (make-parameter #f))
 
@@ -255,7 +249,7 @@
                                    ((string-prefix? command ",show w")
                                     (display %warranty-info)))))))
 (define %help-info
-"Help Commands:
+  "Help Commands:
 
  ,help                    This help.
  ,quit                    Quit this session
@@ -888,7 +882,7 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
         (let ((error (make <queue-full-error> #:ast ast  #:instance instance
                            #:message "queue-full")))
           (clone pc #:status error))
-     (set-deferred (set-state pc (clone state #:q (append q (list trigger)))) instance))))
+        (set-deferred (set-state pc (clone state #:q (append q (list trigger)))) instance))))
 
 (define-method (dequeue (pc <program-counter>))
   (let* ((state (get-state pc))
@@ -903,7 +897,7 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
   (let* ((external-q (.external-q pc))
          (instance (.instance pc))
          (q (or (assoc-ref external-q instance) '())))
-    (if (= (length q) (%queue-size))
+    (if (= (length q) (%queue-size-external))
         (let ((error (make <queue-full-error> #:ast ast #:instance instance
                            #:message "queue-full")))
           (clone pc #:status error))
@@ -1020,8 +1014,8 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
   (set-state pc (clone (get-state pc instance) #:reply '())))
 
 (define-method (reset-reply (pc <program-counter>) (instance <runtime:instance>) (port <string>))
-(let ((reply (.reply (get-state pc instance))))
-  (set-state pc (clone (get-state pc instance) #:reply (alist-delete port reply)))))
+  (let ((reply (.reply (get-state pc instance))))
+    (set-state pc (clone (get-state pc instance) #:reply (alist-delete port reply)))))
 
 (define-method (reset-reply (pc <program-counter>) (port <string>))
   (reset-reply pc (.instance pc) port))
@@ -1109,7 +1103,7 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
          (pc (set-state pc state))
          (error (range-error variable value)))
     (if (not error) pc
-     (clone pc #:status error))))
+        (clone pc #:status error))))
 
 (define-method (assign (pc <program-counter>) (variable <variable>) (e <expression>))
   (next-method))
@@ -1561,7 +1555,7 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
          (pc (make <program-counter> #:id id #:state system-state #:trail trail))
          (pc (if (null? errors) pc
                  (clone pc #:status (car errors)))))
-  pc))
+    pc))
 
 (define-method (make-implicit-illegal (pc <program-counter>) (o <ast>))
   (let ((illegal (make <implicit-illegal-error> #:ast o #:message "illegal")))
